#include "sgg/graphics.h"
#include "brush_utils.h"
#include <iostream>
#include <tuple>

void demonstrateBrushFunctions() {
    // 1. Δημιουργία βασικού Brush με προκαθορισμένα χρώματα και ιδιότητες
    graphics::Brush myBrush = brush_utils::createNewBrush();
    std::cout << "Brush created on default values." << std::endl;

    // 2. Εφαρμογή κύριου χρώματος γεμίσματος (κόκκινο)
    brush_utils::setFillColor(myBrush, 1.0f, 0.0f, 0.0f);
    std::cout << "Main Brush fill color defined on red." << std::endl;

    // 3. Ανάκτηση του κύριου χρώματος γεμίσματος
    //auto [fillR, fillG, fillB] = brush_utils::getFillColor(myBrush);
    std::tuple<float, float, float> fillColor = brush_utils::getFillColor(myBrush);
    float fillR = std::get<0>(fillColor);
    float fillG = std::get<1>(fillColor);
    float fillB = std::get<2>(fillColor);
    std::cout << "Main brush fill color using RGB(" << fillR << ", " << fillG << ", " << fillB << ")" << std::endl;

    // 4. Εφαρμογή δευτερεύοντος χρώματος γεμίσματος για gradient εφέ (μπλε)
    brush_utils::setSecondaryFillColor(myBrush, 0.0f, 0.0f, 1.0f);
    std::cout << "Secondry brush fill color defined on blue." << std::endl;

    // 5. Ανάκτηση του δευτερεύοντος χρώματος γεμίσματος
    //auto [secFillR, secFillG, secFillB] = brush_utils::getFillSecondaryColor(myBrush);
    std::tuple<float, float, float> secFillColor = brush_utils::getFillSecondaryColor(myBrush);
    float secFillR = std::get<0>(secFillColor);
    float secFillG = std::get<1>(secFillColor);
    float secFillB = std::get<2>(secFillColor);

    std::cout << "Secondary brush fill color using RGB(" << secFillR << ", " << secFillG << ", " << secFillB << ")" << std::endl;

    // 6. Εφαρμογή αδιαφάνειας γεμίσματος
    brush_utils::setFillOpacity(myBrush, 0.8f);
    std::cout << "FillOpacity defined 0.8." << std::endl;

    // 7. Ανάκτηση αδιαφάνειας γεμίσματος
    float fillOpacity = brush_utils::getFillOpacity(myBrush);
    std::cout << "FillOpacity is " << fillOpacity << std::endl;

    // 8. Εφαρμογή χρώματος περιγράμματος (πράσινο)
    brush_utils::setOutlineColor(myBrush, 0.0f, 1.0f, 0.0f);
    std::cout << "Brush OutlineColor defined on green." << std::endl;

    // 9. Ανάκτηση του χρώματος περιγράμματος
    //auto [outlineR, outlineG, outlineB] = brush_utils::getOutlineColor(myBrush);
    std::tuple<float, float, float> outlineColor = brush_utils::getOutlineColor(myBrush);
    float outlineR = std::get<0>(outlineColor);
    float outlineG = std::get<1>(outlineColor);
    float outlineB = std::get<2>(outlineColor);

    std::cout << "OutlineColor using RGB(" << outlineR << ", " << outlineG << ", " << outlineB << ")" << std::endl;

    // 10. Εφαρμογή αδιαφάνειας περιγράμματος
    brush_utils::setOutlineOpacity(myBrush, 0.9f);
    std::cout << "OutlineOpacity defined on 0.9." << std::endl;

    // 11. Ανάκτηση της αδιαφάνειας περιγράμματος
    float outlineOpacity = brush_utils::getOutlineOpacity(myBrush);
    std::cout << "Outline Opacity is  " << outlineOpacity << std::endl;

    // 12. Εφαρμογή πλάτους περιγράμματος
    brush_utils::setOutlineWidth(myBrush, 2.0f);
    std::cout << "OutlineWidth defined on 2.0." << std::endl;

    // 13. Ανάκτηση πλάτους περιγράμματος
    float outlineWidth = brush_utils::getOutlineWidth(myBrush);
    std::cout << "OutlineWidth is  " << outlineWidth << std::endl;

    // 14. Εφαρμογή gradient στο Brush
    brush_utils::enableGradient(myBrush, true);
    std::cout << "Brush gradient is enabled ." << std::endl;

    
    // 15. Έλεγχος αν το gradient είναι ενεργοποιημένο
    bool gradientStatus = brush_utils::isGradient(myBrush);
    std::cout << "Brush gradient is enabled : " << (gradientStatus ? "Ναι" : "Όχι") << std::endl;
    

    // 16. Ρύθμιση κατεύθυνσης gradient
    brush_utils::setGradientDirection(myBrush, 1.0f, 0.0f);
    std::cout << "GradientDirection defined on (1.0, 0.0)." << std::endl;

    // 17. Επαναφορά του Brush στις προκαθορισμένες ρυθμίσεις
    brush_utils::resetBrush(myBrush);
    std::cout << "Reset Brush on default." << std::endl;

    
    // 18. Έλεγχος αν το Brush είναι διαφανές
    bool transparencyStatus = brush_utils::isTransparent(myBrush);
    std::cout << "Brush is transparent : " << (transparencyStatus ? "Yes" : "No") << std::endl;
    
    }

int main() {
    graphics::createWindow(800, 600, "Brush Utilities Demo");

    demonstrateBrushFunctions(); // Κλήση της συνάρτησης επίδειξης για τις λειτουργίες του Brush

    graphics::destroyWindow();
    return 0;
}